# -*- coding: utf-8 -*-

#2a) Le sequenze biologiche sono usualmente rappresentate con stringhe, 
# utilizzando un alfabeto diverso a seconda della tipologia di sequenza.
# Alcuni alfabeti sono stati standardizzati; tra questi, abbiamo, per il DNA,
# "unambiguous_dna" (esattamente un simbolo per nucleotide) e "ambiguous_dna" 
# (alcuni simboli indicano la possibilità di avere più tipi diversi di base in 
# una data posizione); analogamente, per l'RNA abbiamo "unambiguous_rna" e 
# "ambiguous_rna"; per le proteine, "protein" (esattamente un simbolo per 
# residuo) ed "extended_protein" (con simboli per casi ambigui e per residui 
# non standard).
# Si noti che "unambiguous_dna" è un sottoinsieme di "ambiguous_dna", che 
# "unambiguous_rna" è un sottoinsieme di "ambiguous_rna", e che "protein" è 
# un sottoinsieme di "extended_protein".
# I simboli degli alfabeti descritti sono contenuti nei file di ugual nome.
# Si scriva una funzione fitting_alpha(s) che, data una stringa che rappresenta 
# una sequenza biologica, restituisca la lista degli alfabeti con il minor 
# numero di simboli a cui essa può corrispondere. Essa dovrà restituire una 
# lista con il solo elemento "unknown" se nessun alfabeto tra quelli descritti 
# è adatto per s. 
# Nella soluzione NON SI DEVONO USARE GLI ALFABETI IUPAC DEFINITI IN BIOPYTHON.


from soluzione1 import file2list

a_dna  = file2list("ambiguous_dna")
a_rna  = file2list("ambiguous_rna")
u_dna  = file2list("unambiguous_dna")
u_rna  = file2list("unambiguous_rna")
e_prot = file2list("extended_protein")
prot   = file2list("protein")

all_al=[(a_dna, len(a_dna), "ambiguous_dna"), \
    (a_rna, len(a_rna), "ambiguous_rna"),     \
    (u_dna, len(u_dna), "unambiguous_dna"),   \
    (u_rna, len(u_rna), "unambiguous_rna"),   \
    (e_prot,len(e_prot),"extended_protein"),  \
    (prot,  len(prot),  "protein")
    ]


def fitting_alpha(s):
    """
    Returns a list with the names of the minimum-size alphabet compatible
    with string s, interpreted as a biosequence
    """
    matching=[]    
    for a in all_al:
        matching.append(a)
        for c in s:
            if c not in a[0]:
                matching.pop()
                break
    if not matching:
        return ["unknown"]
    
    minsize=min(l for a,l,n in matching)
    return [n for a,l,n in matching if l==minsize]
